@@ -19,6 +19,16 @@
 #
 #   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+#tt
+#tt
+#tt
+#tt
+#tt
+#tt
+#tt
+#tt
+
+
 #pod =for test_synopsis my ( %args, $expected_exception, $expected_messages );
 #pod
 #pod =head1 SYNOPSIS
@@ -48,21 +58,49 @@
 #pod         # ^ Construction never fails,
 #pod         #   it always returns an object,
 #pod         #   either builder or survivor.
-#pod     my $ex = exception { $tzil->build(); }; # or ->release();
+#pod     my $ex = exception { $tzil->build(); }; # or $tzil->release();
 #pod         # ^ Builder does build,
 #pod         #   survivor rethrows the saved exception.
 #pod     is( $ex, $expected_exception, 'check status' );
 #pod     cmd_deeply( $tzil->log_messages, $expected_messages, 'check log messages' );
 #pod         # ^ In *any* case we can check log messages.
 #pod
-#pod =head1 NOTES
+#pod =head2 C<Survivor>
+#pod
+#pod C<Survivor> is shortened name of real class. Full class name is
+#pod C<Dist::Zilla::Tester::DieHard::Survivor>.
+#pod
+#pod Following methods can be called on C<Survivor> object: C<clear_log_events>, C<log_events>,
+#pod C<log_messages>.
+#pod
+#pod C<build>, C<release>
+#pod methods rethrow the saved exception.
+#pod
+#pod =note Completeness
 #pod
 #pod Regular C<Dist::Zilla::Tester> (as of v5.039) is not documented, so and I have to study its sources
 #pod to find out features it provides.
 #pod
-#pod I have implemented only part of C<Dist::Zilla::Tester> features, shown in "SYNOPSIS" and
-#pod "DESCRIPTION". C<Minter> is not (yet?) implemented — I do not need it (yet?). Probably there are
-#pod other not (yet?) implemented features I am not aware of.
+#pod I have implemented only part of C<Dist::Zilla::Tester> features, shown in L</"SYNOPSIS"> and
+#pod L</"DESCRIPTION">. C<Minter> is not (yet?) implemented — I do not need it (yet?). Probably there
+#pod are other not (yet?) implemented features I am not aware of.
+#pod
+#pod =note Implementation Detail
+#pod
+#pod Implementation is simpler if C<Survivor> saves not logger, but entire chrome (logger is a part of
+#pod chrome). In such a case C<Survivor> can consume C<Dist::Zilla::Tester::_Role> and get bunch of
+#pod methods "for free".
+#pod
+#pod =note C<most_recent_log_events> Function
+#pod
+#pod C<Dist::Zilla::Tester> 5.040 introduced C<most_recent_log_events> function which can be used to
+#pod retrieve log events even if builder construction failed. However:
+#pod
+#pod =for :list
+#pod 1.  This module was implemented and released before C<DIst::Zilla> 5.040.
+#pod 2.  C<most_recent_log_events> is not documented.
+#pod 3.  Using C<most_recent_log_events> requires revisiting existing test code, while C<DieHard> does
+#pod     not.
 #pod
 #pod =cut
 
@@ -73,7 +111,7 @@ package Dist::Zilla::Tester::DieHard;
 use Moose;
 
 # ABSTRACT: Die hard Dist::Zilla, but save the messages
-our $VERSION = 'v0.5.0'; # VERSION
+our $VERSION = 'v0.6.0'; # VERSION
 our $CLASS = __PACKAGE__;
 
 extends 'Dist::Zilla::Tester';
@@ -96,7 +134,7 @@ sub builder {
 
 no Moose;
 
-__PACKAGE__->meta->make_immutable;
+$CLASS->meta->make_immutable;
 
 # --------------------------------------------------------------------------------------------------
 
@@ -108,7 +146,7 @@ use Moose;
 use namespace::autoclean;
 
 ## no critic ( ProhibitReusedNames )
-our $VERSION = 'v0.5.0'; # VERSION
+our $VERSION = 'v0.6.0'; # VERSION
 our $CLASS = __PACKAGE__;
 ## critic ( ProhibitReusedNames )
 
@@ -117,54 +155,51 @@ extends join( '::', qw{ Dist Zilla Tester _Builder } );
     #   prerequisities, `cpanm` starts downloading, testing and installing `Dist::Zilla`
     #   ignoring the fact that `Dist::Zilla` is already installed.
 
+use Dist::Zilla::Chrome::Test;
 use Try::Tiny;
 
-our $Logger;            ## no critic ( ProhibitPackageVars )
+our $Chrome;                            ## no critic ( ProhibitPackageVars )
 
 around from_config => sub {
     my ( $orig, $self, @args ) = @_;
-    local $Logger;      ## no critic ( RequireInitializationForLocalVars )
+    local $Chrome;                      ## no critic ( RequireInitializationForLocalVars )
     my $builder;
     try {
         #   Try to create original `Dist::Zilla::Tester::_Builder` first.
         $builder = $self->$orig( @args );
     } catch {
         my $ex = $_;
-        #   v DEBUG v DEBUG v DEBUG v DEBUG v DEBUG v DEBUG v DEBUG v DEBUG v DEBUG v DEBUG v DEBUG
-        #   Test fails on some MSWin32 machines because of
-        #       Attribute (logger) does not pass the type constraint because: Validation failed for 'Object' with value undef at constructor Dist::Zilla::Tester::DieHard::Survivor::new
-        #   Let's try to find out why.
-        if ( not defined( $Logger ) ) {
-            warn "$CLASS: Oops, undefined \$Logger";    ## no critic ( RequireCarping )
-            warn "$CLASS: Exception: \"$ex\"";          ## no critic ( RequireCarping )
+        #   If an exception occurs before builder construction (i. e. in
+        #   `Dist::Zilla::Tester::_Builder`'s `around from_config`), `$Chrome` will be undefined.
+        #   Let's crate a new chrome object.
+        if ( not defined( $Chrome ) ) {
+            $Chrome = Dist::Zilla::Chrome::Test->new();
         };
-        #   ^ DEBUG ^ DEBUG ^ DEBUG ^ DEBUG ^ DEBUG ^ DEBUG ^ DEBUG ^ DEBUG ^ DEBUG ^ DEBUG ^ DEBUG
         #   If creation failed due to exception, create stub object instead.
         $builder = Dist::Zilla::Tester::DieHard::Survivor->new(
             exception => $ex,       # Survivor object saves exception
-            logger    => $Logger,   # and logger.
+            chrome    => $Chrome,   # and chrome.
         );
     };
     return $builder;
 };
 
-#   Saving builder's logger is not trivial. `from_config` is a class method. Before
+#   Saving builder's chrome is not trivial. `from_config` is a class method. Before
 #   `$self->$orig()` call the builder does not exist yet, after call the builder does not exist
 #   already. I need to catch the moment when the builder is already born but not yet died.
-#   `BUILD` object method is called right after builder creation.
+#   `BUILDARGS` method is called just before builder creation, so I can steal chrome from builder
+#   constructor arguments.
 #
-#   To pass information (logger reference) between object method `BUILD` and class method
+#   To pass information (chrome reference) between object method `BUILDARGS` and class method
 #   `from_config` I have to use global (class) variable.
 
-sub BUILD {
-    my ( $self ) = @_;
-    #   Builder's `logger` attribute keeps a reference to `Log::Dispatchouli::Proxy`, while I
-    #   need real logger.
-    $Logger = $self->logger->logger;
-    return;
+around BUILDARGS => sub {
+    my ( $orig, $self, @args ) = @_;
+    $Chrome = $args[ 0 ]->{ chrome };
+    return $self->$orig( @args );
 };
 
-__PACKAGE__->meta->make_immutable;
+$CLASS->meta->make_immutable;
 
 };
 
@@ -179,39 +214,35 @@ package Dist::Zilla::Tester::DieHard::Survivor;         ## no critic ( ProhibitM
 use Moose;
 use namespace::autoclean;
 
+with join( '::', qw{ Dist Zilla Tester _Role } );       # Hide from `AutoPrereqs`.
+
 ## no critic ( ProhibitReusedNames )
-our $VERSION = 'v0.5.0'; # VERSION
+our $VERSION = 'v0.6.0'; # VERSION
+our $CLASS = __PACKAGE__;
 ## critic ( ProhibitReusedNames )
 
+has chrome => (
+    isa         => 'Object',
+    is          => 'ro',
+);
+
 has exception => (              # Survivor stores the exception killed the buider
     is          => 'ro',
     required    => 1,
 );
 
-has logger => (                 # and builder logger.
-    is          => 'ro',
-    isa         => 'Object',
-    required    => 1,
-);
-
-#   Survivor mimics builder to some extent. I need only two methods:
-
-sub log_messages {              #   to have access to the log.
+my $rethrow = sub {
     my ( $self ) = @_;
-    return [ map( { $_->{ message } } @{ $self->logger->events } ) ];
+    die $self->exception;               ## no critic ( RequireCarping )
 };
 
-sub build {                     #   Attempt to call `build` on survivor rethrows the exception
-    my ( $self ) = @_;          #   killed the builder.
-    die $self->exception;       ## no critic ( RequireCarping )
+#   Survivor mimics builder to some extent. I need only few methods:
+for my $method ( qw{ build release } ) {
+    no strict 'refs';                   ## no critic ( ProhibitNoStrict )
+    *{ $method } = $rethrow;
 };
 
-sub release {                   #   Attempt to call `release` on survivor rethrows the exception
-    my ( $self ) = @_;          #   killed the builder.
-    die $self->exception;       ## no critic ( RequireCarping )
-};
-
-__PACKAGE__->meta->make_immutable;
+$CLASS->meta->make_immutable;
 
 }
 
@@ -296,14 +327,15 @@ __PACKAGE__->meta->make_immutable;
 #pod         like( $exception, qr{…}, 'status' );
 #pod     };
 #pod
-#pod works but… C<from_config> dies leaving C<$tzil> undefined, and C<log_messages> method call on
-#pod undefined value will definitely fail:
+#pod works but… C<from_config> dies leaving C<$tzil> undefined, C<log_messages> method is called on
+#pod undefined value definitely fails:
 #pod
 #pod     cmd_deeply( $tzil->log_messages, $expected_messages, 'log messages' );
 #pod     #           ^^^^^^^^^^^^^^^^^^^
 #pod     #           Oops: $tzil undefined.
 #pod
-#pod C<Dist::Zilla> dies, and all the messages logged by either C<Dist::Zilla> or its plugins die too.
+#pod C<Dist::Zilla> dies, and all the messages logged by either C<Dist::Zilla> or its plugins are buried
+#pod with C<Dist::Zilla>.
 #pod
 #pod Using C<Dist::Zilla::Tester::DieHard> instead of regular C<Dist::Zilla::Tester> solves this
 #pod problem: even if a plugin throws an exception in constructor, C<< Builder->from_config >> does not
@@ -328,7 +360,7 @@ Dist::Zilla::Tester::DieHard - Die hard Dist::Zilla, but save the messages
 
 =head1 VERSION
 
-Version v0.5.0, released on 2015-10-26 21:38 UTC.
+Version v0.6.0, released on 2015-11-09 21:35 UTC.
 
 =head1 WHAT?
 
@@ -363,13 +395,62 @@ exception. However, such "delayed death" saves log messages for analysis:
         # ^ Construction never fails,
         #   it always returns an object,
         #   either builder or survivor.
-    my $ex = exception { $tzil->build(); }; # or ->release();
+    my $ex = exception { $tzil->build(); }; # or $tzil->release();
         # ^ Builder does build,
         #   survivor rethrows the saved exception.
     is( $ex, $expected_exception, 'check status' );
     cmd_deeply( $tzil->log_messages, $expected_messages, 'check log messages' );
         # ^ In *any* case we can check log messages.
 
+=head2 C<Survivor>
+
+C<Survivor> is shortened name of real class. Full class name is
+C<Dist::Zilla::Tester::DieHard::Survivor>.
+
+Following methods can be called on C<Survivor> object: C<clear_log_events>, C<log_events>,
+C<log_messages>.
+
+C<build>, C<release>
+methods rethrow the saved exception.
+
+=head1 NOTES
+
+=head2 Completeness
+
+Regular C<Dist::Zilla::Tester> (as of v5.039) is not documented, so and I have to study its sources
+to find out features it provides.
+
+I have implemented only part of C<Dist::Zilla::Tester> features, shown in L</"SYNOPSIS"> and
+L</"DESCRIPTION">. C<Minter> is not (yet?) implemented — I do not need it (yet?). Probably there
+are other not (yet?) implemented features I am not aware of.
+
+=head2 Implementation Detail
+
+Implementation is simpler if C<Survivor> saves not logger, but entire chrome (logger is a part of
+chrome). In such a case C<Survivor> can consume C<Dist::Zilla::Tester::_Role> and get bunch of
+methods "for free".
+
+=head2 C<most_recent_log_events> Function
+
+C<Dist::Zilla::Tester> 5.040 introduced C<most_recent_log_events> function which can be used to
+retrieve log events even if builder construction failed. However:
+
+=over 4
+
+=item 1
+
+This module was implemented and released before C<DIst::Zilla> 5.040.
+
+=item 2
+
+C<most_recent_log_events> is not documented.
+
+=item 3
+
+Using C<most_recent_log_events> requires revisiting existing test code, while C<DieHard> does not.
+
+=back
+
 =head1 WHY?
 
 Usually I test my C<Dist::Zilla> plugins in such a way:
@@ -403,14 +484,15 @@ A straightforward attempt to catch exception thrown in C<from_config>:
         like( $exception, qr{…}, 'status' );
     };
 
-works but… C<from_config> dies leaving C<$tzil> undefined, and C<log_messages> method call on
-undefined value will definitely fail:
+works but… C<from_config> dies leaving C<$tzil> undefined, C<log_messages> method is called on
+undefined value definitely fails:
 
     cmd_deeply( $tzil->log_messages, $expected_messages, 'log messages' );
     #           ^^^^^^^^^^^^^^^^^^^
     #           Oops: $tzil undefined.
 
-C<Dist::Zilla> dies, and all the messages logged by either C<Dist::Zilla> or its plugins die too.
+C<Dist::Zilla> dies, and all the messages logged by either C<Dist::Zilla> or its plugins are buried
+with C<Dist::Zilla>.
 
 Using C<Dist::Zilla::Tester::DieHard> instead of regular C<Dist::Zilla::Tester> solves this
 problem: even if a plugin throws an exception in constructor, C<< Builder->from_config >> does not
@@ -418,15 +500,6 @@ die but returns a "survivor" object which can be used to retrieve log messages.
 
 =for test_synopsis my ( %args, $expected_exception, $expected_messages );
 
-=head1 NOTES
-
-Regular C<Dist::Zilla::Tester> (as of v5.039) is not documented, so and I have to study its sources
-to find out features it provides.
-
-I have implemented only part of C<Dist::Zilla::Tester> features, shown in "SYNOPSIS" and
-"DESCRIPTION". C<Minter> is not (yet?) implemented — I do not need it (yet?). Probably there are
-other not (yet?) implemented features I am not aware of.
-
 =for Pod::Coverage builder
 
 =head1 AUTHOR
